#!/usr/bin/env python
# coding: utf-8

from .settings import VERSION


version_info = VERSION.split('.')

__version__ = VERSION
__project__ = PROJECT = __name__
__author__ = AUTHOR = "Kirill Klenov <horneds@gmail.com>"
__license__ = LICENSE = "GNU LGPL"



from os import path as op
import unittest

from .base import BASEDIR, FakeArgs
from zetalibrary.packer import Packer


class LibsTest(unittest.TestCase):
    folder = op.join(BASEDIR, 'libs')

    def test_jquery(self):
        Packer(op.join(self.folder, 'jquery.js'),
                FakeArgs()).pack()
        self.assertEqual(
            open(op.join(self.folder, '_jquery.js')).read(),
            open(op.join(self.folder, '_jquery.js.orig')).read()
        )

    def test_underscore(self):
        Packer(op.join(self.folder, 'underscore.js'),
                FakeArgs()).pack()
        self.assertEqual(
            open(op.join(self.folder, '_underscore.js')).read(),
            open(op.join(self.folder, '_underscore.js.orig')).read()
        )

    def test_blueprint(self):
        Packer(op.join(self.folder, 'blueprint.css'),
                FakeArgs()).pack()
        self.assertEqual(
            open(op.join(self.folder, '_blueprint.css')).read(),
            open(op.join(self.folder, '_blueprint.css.orig')).read()
        )

    def test_boilerplate(self):
        Packer(op.join(self.folder, 'boilerplate.css'),
                FakeArgs()).pack()
        self.assertEqual(
            open(op.join(self.folder, '_boilerplate.css')).read(),
            open(op.join(self.folder, '_boilerplate.css.orig')).read()
        )

    def test_compass(self):
        Packer(op.join(self.folder, 'compass.scss'),
                FakeArgs()).pack()
        self.assertEqual(
            open(op.join(self.folder, '_compass.scss')).read(),
            open(op.join(self.folder, '_compass.scss.orig')).read()
        )



from os import path as op


BASEDIR = op.dirname(__file__)

class FakeArgs():
    def __init__(self, format=None, prefix='_', compress=False, directory=None, output=None):
        self.format = format
        self.prefix = prefix
        self.compress = compress
        self.directory = directory
        self.output = output



from os import path as op, environ, getcwd


VERSION = '0.5.6'
BASEDIR = op.abspath(op.dirname(__file__))
LIBDIR  = op.join(BASEDIR, 'libs')
CUSTOMDIR = environ.get('ZETA_LIBDIR', None)
FORMATS = ['css', 'scss', 'js']
CURRENT_CONFIG = op.join(getcwd(), "zeta.ini")
HOME_CONFIG = op.join(environ.get('HOME', ''), "zeta.ini")
COLORS = dict(
    okgreen = '\033[92m',
    warning = '\033[93m',
    fail = '\033[91m',
    endc = '\033[0m',
)



import re
import urllib2
from os import path as op

from cssmin import cssmin
from jsmin import jsmin
import scss

from zetalibrary.settings import LIBDIR

scss.LOAD_PATHS = LIBDIR

class Parser(object):

    import_re = None
    comment_re = None

    def __init__(self, basedir, compress=False):
        self.compress = compress
        self.basedir = basedir

    @staticmethod
    def read(path, savedir):
        " Read file from path "
        if path.startswith('http://'):
            name = op.basename(path)
            save_path = op.join(savedir, name)
            if not op.exists(save_path):
                src = urllib2.urlopen(path).read()
                try:
                    open(save_path, 'w').write(src)
                except IOError:
                    return src
            path = save_path
        return open(path, 'r').read()

    def parse_path(self, path, parent=None):
        curdir = op.dirname(parent) if parent else self.basedir
        return self.parse_imports(self.read(path, curdir))

    def parse_imports(self, src):
        " Parse imports from source. "
        result = []
        def child(obj):
            result.append(obj.group(1))
        src = self.import_re.sub(child, src)
        return src, result

    def parse_src(self, src, path=None):
        if self.compress:
            src = self.comment_re.sub('', src)
        return src.strip()


class CSSParser(Parser):
    import_re = re.compile(r'^\s*@import +url\(\s*["\']?([^\)\'\"]+)["\']?\s*\)\s*;?\s*$', re.MULTILINE)
    comment_re = re.compile(r'/\*(?:[^*]|\*+[^*/])*\*+/')
    comment_template = '/* %s */\n'
    link_re = re.compile(r'url\(\s*["\']?([^\)\'\"]+)["\']?\)')

    def parse_src(self, src, path=None):
        src = super(CSSParser, self).parse_src(src)
        def links(obj):
            link_path = obj.group(1)
            for ignore in ('data:image', 'http://', 'https://'):
                if link_path.startswith(ignore):
                    return "url(%s)" % link_path
            try:
                url = "url(%s)" % op.relpath(op.join(op.dirname(path), link_path), self.basedir)
                url = url.replace("\\", "/")
                return url
            except OSError:
                return "url(%s)" % link_path

        src = self.link_re.sub(links, src)
        if self.compress:
            src = cssmin(src)
        return src


class SCSSParser(CSSParser):

    def __init__(self, *args, **kwargs):
        super(SCSSParser, self).__init__(*args, **kwargs)
        self.parser = scss.Scss()
        self.parser._scss_opts['compress'] = self.compress

    def parse_src(self, src, path=None):
        src = super(SCSSParser, self).parse_src(src)
        self.parser._scss_files[path] = self.parser.parse_scss_string(path, src)
        self.parser.parse_children()
        self.parser.parse_extends()
        self.parser.manage_order()
        self.parser.parse_properties()
        src = self.parser.create_css(path)
        src = self.parser.post_process(src)
        return src.strip()


class JSParser(Parser):
    import_re = re.compile(r'^require\(\s*[\'\"]([^\'\"]+)[\'\"]\s*\)\s*;?\s*$', re.MULTILINE)
    comment_re = re.compile(r'/\*(?:[^*]|\*+[^*/])*\*+/')
    comment_template = '// %s\n'

    def parse_src(self, src, path=None):
        src = super(JSParser, self).parse_src(src)
        if self.compress:
            src = jsmin(src)
        return src



import unittest
from os import path as op

from .base import FakeArgs, BASEDIR
from zetalibrary.packer import Packer
from zetalibrary.utils import gen_files, gen_frameworks


class TestPacker(unittest.TestCase):
    folder = op.join(BASEDIR, 'common')

    def test_route(self):
        css_file = op.join(self.folder, 'main.css')
        js_file = op.join(self.folder, 'main.js')
        result = list(gen_files(self.folder))
        self.assertTrue(css_file in result)
        self.assertTrue(js_file in result)

    def test_getframework(self):
        frameworks = list(gen_frameworks())
        self.assertTrue(len(frameworks))

    def test_custom_libs(self):
        js_file = op.join(BASEDIR, 'custom', 'custom.js')
        Packer(js_file, FakeArgs(
            directory=op.join(BASEDIR, 'custom'),
            )).pack()
        self.assertTrue(
                'fake' in open(op.join(BASEDIR, 'custom', '_custom.js')).read())

    def test_pack(self):
        css_file = op.join(self.folder, 'main.css')
        Packer(css_file, FakeArgs(
            output=op.join(BASEDIR, 'common', 'output'),
            compress=True)).pack()
        orig = open(op.join(self.folder, '_main.css.orig')).read()
        test = open(op.join(self.folder, 'output/_main.css')).read()
        self.assertEqual(test, orig)

        js_file = op.join(self.folder, 'main.js')
        Packer(js_file, FakeArgs(
            output=op.join(BASEDIR, 'common', 'output'),
            compress=True)).pack()
        orig = open(op.join(self.folder, '_main.js.orig')).read()
        test = open(op.join(self.folder, 'output/_main.js')).read()
        self.assertEqual(test, orig)



import time
import sys
from os import path as op

from argh import ArghParser, command, arg, wrap_errors
from watchdog.observers import Observer

from zetalibrary.settings import BASEDIR, VERSION, CUSTOMDIR
from zetalibrary.utils import ZetaTrick, pack as zeta_pack, gen_frameworks


@command
def shell():
    " A helper command to be used for shell integration "
    print
    print "# Zeta integration "
    print "# ==================== "
    print "source %s" % op.join(BASEDIR, 'shell.sh')
    print


@command
def libs():
    " Show zeta libs "
    for name, description, version, url in gen_frameworks():
        print name
        print ''.join('-' for _ in xrange(len(name)))
        print description.strip('/*\n ')
        print version.strip('/*\n ')
        print url.strip('/*\n ')
        print


@arg('source')
@arg('-p', '--prefix', default="_", help="Save packed files with prefix. Default is '_'")
@arg('-f', '--format', help="Force format (css, js, ...). By default format parse from file extension")
@arg('-c', '--compress', default=False, help="Compress packed sources")
@arg('-d', '--directory', default=CUSTOMDIR, help="Add custom directory for search with prefix: 'zeta://' By default $ZETA_LIBDIR")
@arg('-o', '--output', help="Set output directory path.")
@arg('-s', '--setup-file', help="Configuration ini file")
@wrap_errors(Exception)
def watch(args):
    " Watch directory for changes and auto pack sources "
    assert op.isdir(args.source), "Watch mode allowed only for directories."
    print 'Zeta-library v. %s watch mode' % VERSION
    print '================================'
    print 'Ctrl+C for exit\n'
    observer = Observer()
    handler = ZetaTrick(args=args)
    observer.schedule(handler, args.source, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
        print "\nWatch mode stoped."
    observer.join()


@arg('source')
@arg('-p', '--prefix', default="_", help="Save packed files with prefix. Default is '_'")
@arg('-f', '--format', help="Force format (css, js, ...). By default format parse from file extension")
@arg('-c', '--compress', default=False, help="Compress packed sources")
@arg('-d', '--directory', default=CUSTOMDIR, help="Add custom directory for search with prefix: 'zeta://' By default $ZETA_LIBDIR")
@arg('-o', '--output', help="Set output directory path")
@arg('-s', '--setup-file', help="Configuration ini file")
@wrap_errors(Exception)
def pack(args):
    " Parse file or dir, import css, js code and save with prefix "
    assert op.exists(args.source), "Does not exists: %s" % args.source
    zeta_pack(args)


def main():
    commands = pack, watch, shell, libs
    names = [f.__name__ for f in commands] + [ 'help' ]

    parser = ArghParser()
    parser.add_argument('-v', '--version', action='version', version=VERSION, help='Show zeta version')
    parser.add_commands(commands)
    argv = sys.argv[1:]
    if argv and not argv[0] in names and not argv[0] in ['-v', '--version']:
        argv.insert(0, 'pack')
    parser.dispatch(argv)


if __name__ == "__main__":
    main()



from os import path as op
from sys import stderr, stdout

from zetalibrary.parser import CSSParser, SCSSParser, JSParser
from zetalibrary.settings import LIBDIR
from zetalibrary.utils import color_msg, ZetaError


class Packer(object):

    def __init__(self, path, args):
        self.path = path.rstrip(op.sep)
        self.basedir = op.abspath(op.dirname(path))
        self.args = args
        self.imports = set()
        self.parsers = dict(
            css = CSSParser(self.basedir, compress=self.args.compress),
            scss = SCSSParser(self.basedir, compress=self.args.compress),
            js = JSParser(self.basedir, compress=self.args.compress),
        )

    def pack(self):
        " Pack and save file "
        pack_name = self.args.prefix + op.basename(self.path)
        pack_path = op.join(self.args.output or self.basedir, pack_name)

        self.out("Packing: %s" % self.path)
        self.out("Output: %s" % pack_path)

        if self.args.format:
            ext = self.get_ext(self.path)
            self.parsers[ext] = self.args.format

        out = "".join(self.merge(self.parse(self.path)))
        try:
            open(pack_path, 'w').write(out)
            self.out("Linked file saved as: '%s'." % pack_path)
        except IOError, ex:
            raise ZetaError(ex)

    @staticmethod
    def get_ext(path):
        _, ext = op.splitext(path)
        return ext.lstrip('.').lower()

    def get_parser(self, path):
        return self.parsers.get(self.get_ext(path))

    def parse(self, path, parent=None):
        self.imports.add(path)
        parser = self.get_parser(path)
        curdir = op.dirname(path)
        result = []
        try:
            src, imports = parser.parse_path(path)
            for f in filter(lambda x: not x in self.imports,
                        map(lambda x: op.abspath(op.relpath(op.join(curdir, x))),
                            map(lambda x: self.parse_path(x, curdir), imports))):
                result = result + self.parse(f, parent=path)

            result.append((path, parent, parser, src))
        except IOError, e:
            self.out(str(e), error=True)

        self.out(" * %s" % path)
        return result

    def merge(self, tree):
        for path, parent, parser, src in tree:
            src = parser.parse_src(src, path)
            if not src:
                continue

            if not self.args.compress:
                yield parser.comment_template % ("=" * 10)
                if parent:
                    yield parser.comment_template % "From: '%s'" % parent.replace(LIBDIR, 'zeta://')
                yield parser.comment_template % "Zeta import: '%s'" % path.replace(LIBDIR, 'zeta://')
                yield src
                yield '\n\n'
            else:
                yield src

        yield '\n'

    def parse_path(self, path, curdir):
        " Normilize path. "
        if path.startswith('http://'):
            return path

        elif path.startswith('zeta://'):
            zpath = op.join(LIBDIR, path[len('zeta://'):])
            if self.args.directory and not op.exists(zpath):
                return op.join(self.args.directory, path[len('zeta://'):])
            return zpath

        return op.abspath(op.normpath(op.join(curdir, path)))

    @staticmethod
    def out(msg, error=False):
        " Send message to shell "
        pipe = stdout
        if error:
            pipe = stderr
            msg = color_msg(msg, "warning")

        pipe.write("%s\n" % msg)



from os import path as op, listdir

from watchdog.tricks import Trick

from zetalibrary.settings import COLORS, FORMATS, LIBDIR, CURRENT_CONFIG, HOME_CONFIG
from ConfigParser import ConfigParser


def color_msg(msg, color):
    " Return colored message "
    return ''.join((COLORS.get(color, COLORS['endc']), msg, COLORS['endc']))


def is_parsed_file(path, prefix="_"):
    name, ext = op.splitext(op.basename(path))
    return op.isfile(path) and not name.startswith(prefix) and ext[1:].lower() in FORMATS


def gen_files(path, prefix="_"):
    " Return file generator "

    if op.isdir(path):
        for name in listdir(path):
            fpath = op.join(path, name)
            if is_parsed_file(fpath):
                yield op.abspath(fpath)

    elif is_parsed_file(path):
        yield op.abspath(path)


def gen_frameworks():
    for fname in sorted(listdir(LIBDIR)):
        name, ext = op.splitext(fname)
        fpath = op.join(LIBDIR, fname)
        if not name.startswith('_') and op.isfile(fpath) and ext.strip('.') in ['css', 'js', 'scss']:
            description, url, version = open(fpath).readlines()[0:3]
            yield (fname, description, version, url)


def pack(args):
    from zetalibrary.packer import Packer
    args = parse_config(args)
    for path in gen_files(args.source, prefix=args.prefix):
        Packer(path, args).pack()


def parse_config(args):
    parser = ConfigParser()
    parser.add_section('Zeta')
    parser.read([CURRENT_CONFIG, HOME_CONFIG, args.setup_file or ''])
    for k,v in parser._sections['Zeta'].iteritems():
        if getattr(args, k, None) is None:
            setattr(args, k, v)
    return args


class ZetaTrick(Trick):
    " Zeta directory event handler "

    def __init__(self, args=None):
        self.args = args
        self.formats = ['css', 'js', 'scss', args.format]
        super(ZetaTrick, self).__init__()

    def dispatch(self, event):
        name = op.basename(event.src_path)
        _, ext = op.splitext(name)
        if (not name.startswith(self.args.prefix)
                and not event.is_directory
                and ext.lstrip('.').lower() in self.formats):
            super(ZetaTrick, self).dispatch(event)


    def on_any_event(self, event):
        print "\nChanges found: %s" % event.src_path
        pack(self.args)


class ZetaError(Exception):
    " Zeta-library error "
    pass



import unittest
from os import path as op

from .base import BASEDIR, FakeArgs
from zetalibrary.packer import Packer


class TestZeta(unittest.TestCase):
    folder = op.join(BASEDIR, 'zeta')

    def test_zeta(self):
        css_file = op.join(self.folder, 'main.css')
        packer = Packer(css_file, FakeArgs())
        packer.pack()
        orig = open(op.join(self.folder, '_main.css.orig')).read()
        test = open(op.join(self.folder, '_main.css')).read()
        self.assertEqual(test, orig)

        js_file = op.join(self.folder, 'main.js')
        packer = Packer(js_file, FakeArgs())
        packer.pack()
        orig = open(op.join(self.folder, '_main.js.orig')).read()
        test = open(op.join(self.folder, '_main.js')).read()
        self.assertEqual(test, orig)
